from typing import Dict, Optional, Tuple

from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import action_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset


class EgfrDropNotificationAdminMixin:
    radio_fields = {
        "report_status": admin.VERTICAL,
        "creatinine_units": admin.VERTICAL,
    }

    def get_fieldsets(
        self, request, obj=None
    ) -> Tuple[Tuple[Optional[str], Dict[str, Tuple[str, ...]]], ...]:
        fieldsets = [
            (None, {"fields": ("subject_visit", "report_datetime")}),
            (
                "eGFR",
                {
                    "fields": (
                        "egfr_percent_change",
                        "creatinine_date",
                        "creatinine_value",
                        "creatinine_units",
                    )
                },
            ),
            (
                "Narrative",
                {"fields": ("narrative", "report_status")},
            ),
            crf_status_fieldset,
        ]
        if getattr(self.model, "action_name", None):
            fieldsets.append(action_fieldset_tuple)
        fieldsets.append(audit_fieldset_tuple)
        return tuple(fieldsets)

    def get_search_fields(self, request) -> Tuple[str, ...]:
        fields = super().get_search_fields(request)
        custom_fields = (
            "subject_visit__subject_identifier",
            "action_identifier",
            "tracking_identifier",
        )
        return tuple(f for f in fields if f not in custom_fields) + custom_fields

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        fields = super().get_readonly_fields(request, obj)
        return (
            "creatinine_date",
            "creatinine_value",
            "creatinine_units",
            "egfr_percent_change",
        ) + fields

    def get_list_filter(self, request) -> Tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        return ("report_status", "creatinine_date") + list_filter

    def get_list_display(self, request) -> Tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = (
            "report_status",
            "report_datetime",
            "egfr_percent_change",
            "creatinine_date",
            "creatinine_value",
        )
        return tuple(f for f in list_display if f not in custom_fields) + custom_fields



from django.contrib import admin

from edc_egfr.admin import EgfrDropNotificationAdminMixin

from .models import EgfrDropNotification


@admin.register(EgfrDropNotification)
class EgfrDropNotificationAdmin(EgfrDropNotificationAdminMixin, admin.ModelAdmin):
    pass



from django.db import models
from edc_constants.constants import COMPLETE, INCOMPLETE, NEW, OPEN
from edc_lab.choices import SERUM_CREATININE_UNITS
from edc_lab_panel.model_mixin_factory import reportable_result_model_mixin_factory
from edc_model import REPORT_STATUS
from edc_reportable.units import EGFR_UNITS, MICROMOLES_PER_LITER
from edc_vitals.models import WeightField


class EgfrDropNotificationModelMixin(
    reportable_result_model_mixin_factory(
        utest_id="egfr",
        verbose_name="eGFR",
        decimal_places=4,
        default_units=EGFR_UNITS,
        max_digits=8,
        units_choices=((EGFR_UNITS, EGFR_UNITS),),
        exclude_attrs_for_reportable=True,
    ),
    reportable_result_model_mixin_factory(
        utest_id="creatinine",
        verbose_name="Serum creatinine",
        decimal_places=2,
        default_units=MICROMOLES_PER_LITER,
        max_digits=8,
        units_choices=((SERUM_CREATININE_UNITS, SERUM_CREATININE_UNITS),),
        exclude_attrs_for_reportable=True,
    ),
    models.Model,
):
    creatinine_date = models.DateField(verbose_name="Serum creatinine date")

    weight = WeightField(null=True, blank=True)

    egfr_percent_change = models.DecimalField(
        verbose_name="Percent change from baseline",
        max_digits=10,
        decimal_places=2,
        null=True,
        blank=True,
        help_text="Copied from RFT result eGFR section.",
    )

    narrative = models.TextField(
        verbose_name="Narrative",
        null=True,
        blank=True,
    )

    report_status = models.CharField(max_length=15, choices=REPORT_STATUS, default=NEW)

    def save(self, *args, **kwargs):
        if self.report_status == OPEN:
            self.crf_status = INCOMPLETE
        else:
            self.crf_status = COMPLETE
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "eGFR Drop Notification"
        verbose_name_plural = "eGFR Drop Notifications"
        abstract = True



from django.apps import apps as django_apps
from django.conf import settings


def get_egfr_drop_notification_model():
    return getattr(settings, "EDC_EGFR_DROP_NOTIFICATION_MODEL")


def get_egfr_drop_notification_model_cls():
    return django_apps.get_model(get_egfr_drop_notification_model())



from datetime import date

from django.db import models
from django.db.models import PROTECT
from edc_consent.model_mixins import ConsentVersionModelMixin
from edc_constants.choices import GENDER
from edc_crf.model_mixins import CrfStatusModelMixin
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierModelMixin
from edc_lab.model_mixins import PanelModelMixin
from edc_lab_panel.panels import rft_panel
from edc_lab_results.model_mixins import BloodResultsMethodsModelMixin
from edc_model.models import BaseUuidModel
from edc_registration.model_mixins import UpdatesOrCreatesRegistrationModelMixin
from edc_reportable import MICROMOLES_PER_LITER
from edc_screening.model_mixins import ScreeningIdentifierModelMixin
from edc_sites.model_mixins import SiteModelMixin
from edc_utils import get_utcnow
from edc_visit_tracking.models import SubjectVisit

from edc_egfr.model_mixins import EgfrDropNotificationModelMixin, EgfrModelMixin


class SubjectScreening(ScreeningIdentifierModelMixin, BaseUuidModel):
    screening_identifier = models.CharField(
        verbose_name="Screening ID",
        max_length=50,
        blank=True,
        unique=True,
        editable=False,
    )

    gender = models.CharField(choices=GENDER, max_length=10)

    age_in_years = models.IntegerField()

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time",
        default=get_utcnow,
        help_text="Date and time of report.",
    )


class SubjectConsent(
    SiteModelMixin,
    ConsentVersionModelMixin,
    NonUniqueSubjectIdentifierModelMixin,
    UpdatesOrCreatesRegistrationModelMixin,
    BaseUuidModel,
):
    consent_datetime = models.DateTimeField(default=get_utcnow)

    identity = models.CharField(max_length=25)

    confirm_identity = models.CharField(max_length=25)

    dob = models.DateField(default=date(1995, 1, 1))


class SubjectRequisition(PanelModelMixin, BaseUuidModel):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    subject_identifier = models.CharField(max_length=25, null=True)

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time",
        default=get_utcnow,
        help_text="Date and time of report.",
    )

    requisition_datetime = models.DateTimeField(
        default=get_utcnow,
        verbose_name="Requisition Date",
    )

    class Meta:
        pass


class ResultCrf(BloodResultsMethodsModelMixin, EgfrModelMixin, models.Model):
    lab_panel = rft_panel

    egfr_formula_name = "ckd-epi"

    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    requisition = models.ForeignKey(SubjectRequisition, on_delete=PROTECT)

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time",
        default=get_utcnow,
        help_text="Date and time of report.",
    )

    assay_datetime = models.DateTimeField(default=get_utcnow())

    creatinine_value = models.DecimalField(
        decimal_places=2, max_digits=6, null=True, blank=True
    )

    creatinine_units = models.CharField(
        verbose_name="units",
        max_length=10,
        choices=((MICROMOLES_PER_LITER, MICROMOLES_PER_LITER),),
        null=True,
        blank=True,
    )

    @property
    def related_visit(self):
        return self.subject_visit


class EgfrDropNotification(
    SiteModelMixin, CrfStatusModelMixin, EgfrDropNotificationModelMixin, BaseUuidModel
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=get_utcnow
    )

    consent_version = models.CharField(max_length=5, default="1")

    class Meta(EgfrDropNotificationModelMixin.Meta, BaseUuidModel.Meta):
        app_label = "egfr_app"



from typing import Any

from django import forms
from edc_reportable import CalculatorError, ConversionNotHandled

from ..calculators import EgfrCkdEpi, EgfrCockcroftGault


class EgfrCkdEpiFormValidatorMixin:
    def validate_egfr(self: Any):
        if (
            self.cleaned_data.get("gender")
            and self.cleaned_data.get("age_in_years")
            and self.cleaned_data.get("ethnicity")
            and self.cleaned_data.get("creatinine_value")
            and self.cleaned_data.get("creatinine_units")
        ):
            opts = dict(
                gender=self.cleaned_data.get("gender"),
                age_in_years=self.cleaned_data.get("age_in_years"),
                ethnicity=self.cleaned_data.get("ethnicity"),
                creatinine_value=self.cleaned_data.get("creatinine_value"),
                creatinine_units=self.cleaned_data.get("creatinine_units"),
            )
            try:
                egfr = EgfrCkdEpi(**opts).value
            except (CalculatorError, ConversionNotHandled) as e:
                raise forms.ValidationError(e)
            return egfr
        return None


class EgfrCockcroftGaultFormValidatorMixin:
    def validate_egfr(self: Any):
        if (
            self.cleaned_data.get("gender")
            and self.cleaned_data.get("age_in_years")
            and self.cleaned_data.get("weight")
            and self.cleaned_data.get("creatinine_value")
            and self.cleaned_data.get("creatinine_units")
        ):
            opts = dict(
                gender=self.cleaned_data.get("gender"),
                age_in_years=self.cleaned_data.get("age_in_years"),
                weight=self.cleaned_data.get("weight"),
                creatinine_value=self.cleaned_data.get("creatinine_value"),
                creatinine_units=self.cleaned_data.get("creatinine_units"),
            )
            try:
                egfr = EgfrCockcroftGault(**opts).value
            except (CalculatorError, ConversionNotHandled) as e:
                raise forms.ValidationError(e)
            return egfr
        return None
